package com.zanxiang.sdk.service.Impl;

import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.zanxiang.advertising.oceanengine.track.base.pojo.PlatformServer;
import com.zanxiang.advertising.oceanengine.track.base.pojo.dto.AppOrderRpcDTO;
import com.zanxiang.advertising.oceanengine.track.base.pojo.dto.UserRegisterRpcDTO;
import com.zanxiang.advertising.oceanengine.track.base.pojo.rpc.IAppOrderRpc;
import com.zanxiang.advertising.oceanengine.track.base.pojo.rpc.IAppUserRpc;
import com.zanxiang.advertising.oceanengine.track.base.pojo.vo.AppReportLogRpcVO;
import com.zanxiang.common.enums.CallBackEnum;
import com.zanxiang.module.util.JsonUtil;
import com.zanxiang.module.util.pojo.ResultVO;
import com.zanxiang.mybatis.entity.Order;
import com.zanxiang.mybatis.entity.User;
import com.zanxiang.sdk.domain.dto.PlatformOrderDTO;
import com.zanxiang.sdk.enums.OsTypeEnum;
import com.zanxiang.sdk.enums.PayTypeEnum;
import com.zanxiang.sdk.service.CallBackService;
import com.zanxiang.sdk.service.OrderService;
import com.zanxiang.sdk.service.UserService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.DubboReference;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;

/**
 * @author : lingfeng
 * @time : 2022-11-10
 * @description : 回传
 */
@Slf4j
@Service
public class CallBackServiceImpl implements CallBackService {

    @Autowired
    private UserService userService;

    @Autowired
    private OrderService orderService;

    @DubboReference(providedBy = PlatformServer.SERVER_DUBBO_NAME)
    private IAppUserRpc appUserRpc;

    @DubboReference(providedBy = PlatformServer.SERVER_DUBBO_NAME)
    private IAppOrderRpc appOrderRpc;

    /**
     * 用户回传
     *
     * @param user 用户
     */
    @Override
    public void userCallBack(User user) {
        if (user == null) {
            return;
        }
        log.error("用户注册回传, userId : {}", user.getId());
        UserRegisterRpcDTO dto = UserRegisterRpcDTO.builder()
                .appKey(String.valueOf(user.getGameId()))
                .userId(String.valueOf(user.getId()))
                .username(user.getUsername())
                .mac(user.getMac())
                .imei(user.getImei())
                .androidId(user.getAndroidId())
                .ip(user.getIp())
                .osType(OsTypeEnum.getByName(user.getDeviceSystem()))
                .registerTime(user.getCreateTime())
                .build();
        //调接口
        ResultVO<AppReportLogRpcVO> result;
        try {
            result = appUserRpc.register(dto);
        } catch (Exception e) {
            log.error("用户注册回传上报异常, user : {}, e : {}", JsonUtil.toString(user), e.getMessage());
            return;
        }
        log.error("用户注册回传结果, userId : {}, result : {}", user.getId(), JsonUtil.toString(result));
        String adId = "AD_ID";
        //成功
        if (result.isSuccess() && result.getData() != null && result.getData().getAid() != null) {
            adId = result.getData().getAid().toString();
        }
        //更新数据库
        userService.update(new LambdaUpdateWrapper<User>().set(User::getAdId, adId).eq(User::getId, user.getId()));
    }

    /**
     * 订单回传
     *
     * @param orderInfo 订单信息
     */
    @Override
    public void orderCallBack(PlatformOrderDTO orderInfo) {
        if (orderInfo == null) {
            return;
        }
        log.error("用户注册回传, orderId : {}", orderInfo.getOrderId());
        AppOrderRpcDTO dto = AppOrderRpcDTO.builder()
                .appKey(String.valueOf(orderInfo.getGameId()))
                .orderId(orderInfo.getOrderId())
                .userId(String.valueOf(orderInfo.getUserId()))
                .amount(orderInfo.getAmount().multiply(new BigDecimal("100")).longValue())
                .realAmount(orderInfo.getRealAmount().multiply(new BigDecimal("100")).longValue())
                .tranNo(orderInfo.getMerchantOrderNo())
                .orderStatus(orderInfo.getStatus())
                .payType(PayTypeEnum.getByPayType(orderInfo.getPayWayId()))
                .payTime(orderInfo.getPayTime())
                .createTime(orderInfo.getCreateTime())
                .build();
        //调接口
        ResultVO<AppReportLogRpcVO> result;
        try {
            result = appOrderRpc.order(dto);
        } catch (Exception e) {
            log.error("订单回传上报异常, orderInfo : {}, e : {}", JsonUtil.toString(orderInfo), e.getMessage());
            return;
        }
        log.error("订单回传结果, orderId : {}, result : {}", orderInfo.getOrderId(), JsonUtil.toString(result));
        String adId = "AD_ID";
        Integer callBackStatus = CallBackEnum.FAIL_CALL_BACK.getCallBackStatus();
        //成功
        if (result.isSuccess() && result.getData() != null && result.getData().getAid() != null) {
            adId = result.getData().getAid().toString();
            callBackStatus = CallBackEnum.SUCCESS_CALL_BACK.getCallBackStatus();
        }
        //更新数据库
        orderService.update(new LambdaUpdateWrapper<Order>()
                .set(Order::getAdId, adId)
                .set(Order::getCallBackStatus, callBackStatus)
                .eq(Order::getOrderId, orderInfo.getOrderId()));
    }

}
